@@ -43,7 +43,7 @@ from keras_cv_attention_models.cmt.cmt import light_mhsa_with_multi_head_relativ
 from keras_cv_attention_models.uniformer.uniformer import multi_head_self_attention
 from keras_cv_attention_models.davit.davit import multi_head_self_attention_channel, window_attention
 from keras_cv_attention_models.swin_transformer_v2.swin_transformer_v2 import (
-    DivideScale,
+    ExpLogitScale,
     PairWiseRelativePositionalEmbedding,
     shifted_window_attention,
     WindowAttentionMask,
