@@ -212,10 +212,11 @@ def batchnorm_with_activation(
     return nn
 
 
-def layer_norm(inputs, epsilon=LAYER_NORM_EPSILON, name=None):
+def layer_norm(inputs, zero_gamma=False, epsilon=LAYER_NORM_EPSILON, name=None):
     """ Typical LayerNormalization with epsilon=1e-5 """
     norm_axis = -1 if K.image_data_format() == "channels_last" else 1
-    return keras.layers.LayerNormalization(axis=norm_axis, epsilon=epsilon, name=name and name + "ln")(inputs)
+    gamma_initializer = tf.zeros_initializer() if zero_gamma else tf.ones_initializer()
+    return keras.layers.LayerNormalization(axis=norm_axis, epsilon=epsilon, gamma_initializer=gamma_initializer, name=name and name + "ln")(inputs)
 
 
 def group_norm(inputs, groups=32, epsilon=BATCH_NORM_EPSILON, name=None):
