@@ -1,7 +1,8 @@
 import tensorflow as tf
 from tensorflow import keras
 from keras_cv_attention_models.attention_layers import (
-    ChannelAffine,
+    BiasLayer,
+    # ChannelAffine,
     drop_block,
     layer_norm,
     mlp_block,
@@ -11,17 +12,26 @@ from keras_cv_attention_models.attention_layers import (
 from keras_cv_attention_models.download_and_load import reload_model_weights
 
 PRETRAINED_DICT = {
-    "swin_transformer_v2_tiny_ns": {"imagenet": {224: "c3272af88ba0cf09c818ac558ca9970e"}},
-    "swin_transformer_v2_small": {"imagenet": {224: "d885a15b6d19cf72eee3a43d9c548579"}},
-    "swin_transformer_v2_small_ns": {"imagenet": {224: "89d5a63d528bbb88a4a287871e868414"}},
+    "swin_transformer_v2_base_window12": {"imagenet21k": {192: "b68add0f06aeede5d4dd10cd26855f36"}},
+    "swin_transformer_v2_base_window16": {"imagenet22k": {256: "1caba96ed702d467e650465503033c85"}},
+    "swin_transformer_v2_base_window16": {"imagenet": {256: "a7bfb9ae0733807baa702d05eb8feab8"}},
+    "swin_transformer_v2_base_window24": {"imagenet22k": {384: "acd467e1d8555e15542c8254bdae1b72"}},
+    "swin_transformer_v2_base_window8": {"imagenet": {256: "15454d9f6ba2ccca940f9c45b6935af6"}},
+    "swin_transformer_v2_large_window12": {"imagenet21k": {192: "ace20b0d634eb92989ece52e300440d5"}},
+    "swin_transformer_v2_large_window16": {"imagenet22k": {256: "151bb82a138f956613ce4b9885bfdd18"}},
+    "swin_transformer_v2_large_window24": {"imagenet22k": {384: "04fa3b195e5201c2d1068d1e19c8a0c5"}},
+    "swin_transformer_v2_small_window16": {"imagenet": {256: "3b2ca43d1927cca1b414b60e1044a84d"}},
+    "swin_transformer_v2_small_window8": {"imagenet": {256: "0a8468bd9acdf2056fc401e9f5067f97"}},
+    "swin_transformer_v2_tiny_window16": {"imagenet": {256: "37ce8c5f514c2249ef10d9c3acc37d29"}},
+    "swin_transformer_v2_tiny_window8": {"imagenet": {256: "9317f155e37e4081a09d290ca99bf7cd"}},
 }
 
 
-@tf.keras.utils.register_keras_serializable(package="swinv2")
-class DivideScale(keras.layers.Layer):
-    def __init__(self, axis=-1, initializer="ones", min_value=0.01, **kwargs):
+@tf.keras.utils.register_keras_serializable(package="kecam")
+class ExpLogitScale(keras.layers.Layer):
+    def __init__(self, axis=-1, init_value=10.0, max_value=100.0, **kwargs):
         super().__init__(**kwargs)
-        self.axis, self.initializer, self.min_value = axis, initializer, min_value
+        self.axis, self.init_value, self.max_value = axis, init_value, max_value
 
     def build(self, input_shape):
         if self.axis == -1 or self.axis == len(input_shape) - 1:
@@ -31,41 +41,74 @@ class DivideScale(keras.layers.Layer):
             axis = self.axis if isinstance(self.axis, (list, tuple)) else [self.axis]
             for ii in axis:
                 weight_shape[ii] = input_shape[ii]
-        self.scale = self.add_weight(name="weight", shape=weight_shape, initializer=self.initializer, trainable=True)
+
+        initializer = tf.initializers.constant(tf.math.log(self.init_value))
+        self.scale = self.add_weight(name="weight", shape=weight_shape, initializer=initializer, trainable=True)
+        self.__max_value__ = tf.math.log(self.max_value)
         super().build(input_shape)
 
     def call(self, inputs, **kwargs):
-        return inputs / tf.maximum(self.scale, self.min_value)
+        return inputs * tf.math.exp(tf.minimum(self.scale, self.__max_value__))
 
     def get_config(self):
         config = super().get_config()
-        config.update({"axis": self.axis, "min_value": self.min_value})  # Not saving initializer in config
+        config.update({"axis": self.axis, "max_value": self.max_value})
         return config
 
 
-@tf.keras.utils.register_keras_serializable(package="swinv2")
+@tf.keras.utils.register_keras_serializable(package="kecam")
 class PairWiseRelativePositionalEmbedding(keras.layers.Layer):
-    def __init__(self, **kwargs):
+    def __init__(self, pos_scale=-1, **kwargs):
         # No weight, just need to wrapper a layer, or will not in model structure
+        self.pos_scale = pos_scale
         super().__init__(**kwargs)
 
+    def __build_pairwise_relative_position_index__(self, input_shape):
+        # input_shape: [batch * window_patch, window_height, window_width, channel]
+        height, width = input_shape[1], input_shape[2]  # [12, 15]
+        hh, ww = tf.meshgrid(range(height), range(width))
+        coords = tf.stack([hh, ww], axis=-1)  # [15, 12, 2]
+        coords_flatten = tf.reshape(coords, [-1, 2])  # [180, 2]
+        relative_coords = coords_flatten[:, None, :] - coords_flatten[None, :, :]  # [180, 180, 2]
+        # relative_coords = tf.reshape(relative_coords, [-1, 2])  # [196 * 196, 2]
+
+        relative_coords_hh = relative_coords[:, :, 0] + height - 1
+        relative_coords_ww = (relative_coords[:, :, 1] + width - 1) * (2 * height - 1)
+        relative_coords_hhww = tf.stack([relative_coords_hh, relative_coords_ww], axis=-1)
+        self.relative_position_index = tf.reduce_sum(relative_coords_hhww, axis=-1)  # [180, 180]
+
+    def __build_relative_coords_table__(self, input_shape):
+        # input_shape: [batch * window_patch, window_height, window_width, channel]
+        height, width = input_shape[1], input_shape[2]  # [12, 15]
+        hh, ww = tf.meshgrid(range(-height + 1, height), range(-width + 1, width), indexing="ij")
+        coords = tf.cast(tf.stack([hh, ww], axis=-1), self.dtype)
+        if self.pos_scale == -1:
+            pos_scale = [height, width]
+        else:
+            # If pretrined weights are from different input_shape or window_size, pos_scale is previous actually using window_size
+            pos_scale = self.pos_scale if isinstance(self.pos_scale, (list, tuple)) else [self.pos_scale, self.pos_scale]
+        coords = coords * 8 / [float(pos_scale[0] - 1), float(pos_scale[1] - 1)]  # [23, 29, 2], normalize to -8, 8
+        # torch.sign(relative_coords_table) * torch.log2(torch.abs(relative_coords_table) + 1.0) / math.log2(8)
+        relative_log_coords = tf.sign(coords) * tf.math.log(1.0 + tf.abs(coords)) / (tf.math.log(2.0) * 3.0)
+        self.relative_log_coords = tf.reshape(relative_log_coords, [-1, 2])  # [23 * 29, 2]
+        self.height, self.width = height, width  # For reload with shape mismatched
+
     def build(self, input_shape):
         # input_shape: [batch * window_patch, window_height, window_width, channel]
-        height, width = input_shape[1], input_shape[2]
-        xx, yy = tf.meshgrid(range(height), range(width))  # tf.meshgrid is same with np.meshgrid 'xy' mode, while torch.meshgrid 'ij' mode
-        coords = tf.stack([yy, xx], axis=-1)  # [14, 14, 2]
-        coords_flatten = tf.reshape(coords, [-1, 2])  # [196, 2]
-        relative_coords = coords_flatten[:, None, :] - coords_flatten[None, :, :]  # [196, 196, 2]
-        # relative_coords = tf.reshape(relative_coords, [-1, 2])  # [196 * 196, 2]
-        relative_coords = tf.cast(relative_coords, self.dtype)
-        self.relative_coords_log = tf.sign(relative_coords) * tf.math.log(1.0 + tf.abs(relative_coords))
+        self.__build_relative_coords_table__(input_shape)
+        self.__build_pairwise_relative_position_index__(input_shape)
         super().build(input_shape)
 
     def call(self, inputs, **kwargs):
-        return self.relative_coords_log
+        return self.relative_log_coords, self.relative_position_index
+
+    def get_config(self):
+        base_config = super().get_config()
+        base_config.update({"pos_scale": self.pos_scale})
+        return base_config
 
 
-@tf.keras.utils.register_keras_serializable(package="swinv2")
+@tf.keras.utils.register_keras_serializable(package="kecam")
 class WindowAttentionMask(keras.layers.Layer):
     def __init__(self, height, width, window_height, window_width, shift_height=0, shift_width=0, **kwargs):
         # No weight, just need to wrapper a layer, or will meet some error in model saving or loading...
@@ -119,27 +162,36 @@ class WindowAttentionMask(keras.layers.Layer):
 
 
 def window_mhsa_with_pair_wise_positional_embedding(
-    inputs, num_heads=4, key_dim=0, meta_hidden_dim=384, mask=None, out_bias=True, attn_dropout=0, out_dropout=0, name=None
+    inputs, num_heads=4, key_dim=0, meta_hidden_dim=512, mask=None, pos_scale=-1, out_bias=True, qv_bias=True, attn_dropout=0, out_dropout=0, name=None
 ):
     input_channel = inputs.shape[-1]
     key_dim = key_dim if key_dim > 0 else input_channel // num_heads
     qk_out = key_dim * num_heads
 
-    qkv = keras.layers.Dense(qk_out * 3, use_bias=True, name=name and name + "qkv")(inputs)
+    qkv = keras.layers.Dense(qk_out * 3, use_bias=False, name=name and name + "qkv")(inputs)
     qkv = tf.reshape(qkv, [-1, qkv.shape[1] * qkv.shape[2], qkv.shape[-1]])
     query, key, value = tf.split(qkv, 3, axis=-1)
-    query = tf.transpose(tf.reshape(query, [-1, query.shape[1], num_heads, key_dim]), [0, 2, 1, 3])  #  [batch, num_heads, hh * ww, key_dim]
+    if qv_bias:
+        query = BiasLayer(name=name and name + "query_bias")(query)
+        value = BiasLayer(name=name and name + "value_bias")(value)
+    query = tf.transpose(tf.reshape(query, [-1, query.shape[1], num_heads, key_dim]), [0, 2, 1, 3])  # [batch, num_heads, hh * ww, key_dim]
     key = tf.transpose(tf.reshape(key, [-1, key.shape[1], num_heads, key_dim]), [0, 2, 3, 1])  # [batch, num_heads, key_dim, hh * ww]
     value = tf.transpose(tf.reshape(value, [-1, value.shape[1], num_heads, key_dim]), [0, 2, 1, 3])  # [batch, num_heads, hh * ww, vv_dim]
 
+    # cosine attention
     norm_query, norm_key = tf.nn.l2_normalize(query, axis=-1, epsilon=1e-6), tf.nn.l2_normalize(key, axis=-2, epsilon=1e-6)
-    attn = tf.matmul(norm_query, norm_key)
-    attn = DivideScale(axis=1, name=name and name + "scale")(attn)  # axis=1 means on head dimension
+    attn = tf.matmul(norm_query, norm_key)  # [batch, num_heads, hh * ww, hh * ww]
+    attn = ExpLogitScale(axis=1, name=name and name + "scale")(attn)  # axis=1 is head dimension
 
-    # _relative_positional_encodings
-    pos_coord = PairWiseRelativePositionalEmbedding(name=name and name + "pos_emb")(inputs)  # Wrapper a layer, or will not in model structure
-    relative_position_bias = mlp_block(pos_coord, meta_hidden_dim, output_channel=num_heads, drop_rate=0.1, activation="relu", name=name and name + "meta_")
-    relative_position_bias = tf.expand_dims(tf.transpose(relative_position_bias, [2, 0, 1]), 0)
+    # PairWiseRelativePositionalEmbedding -> mlp -> add with attn
+    pos_coords, pos_index = PairWiseRelativePositionalEmbedding(pos_scale=pos_scale, name=name and name + "pos_emb")(inputs)
+    relative_position_bias = keras.layers.Dense(meta_hidden_dim, use_bias=True, name=name and name + "meta_dense_1")(pos_coords)
+    relative_position_bias = keras.layers.Activation("relu")(relative_position_bias)
+    relative_position_bias = keras.layers.Dense(num_heads, use_bias=False, name=name and name + "meta_dense_2")(relative_position_bias)
+
+    relative_position_bias = tf.gather(relative_position_bias, pos_index)  # [hh * ww, hh * ww, num_heads]
+    relative_position_bias = tf.nn.sigmoid(relative_position_bias) * 16.0
+    relative_position_bias = tf.expand_dims(tf.transpose(relative_position_bias, [2, 0, 1]), 0)  # [1, num_heads, hh * ww, hh * ww]
     attn = attn + relative_position_bias
 
     if mask is not None:
@@ -159,7 +211,7 @@ def window_mhsa_with_pair_wise_positional_embedding(
     return attention_output
 
 
-def shifted_window_attention(inputs, window_size, num_heads=4, shift_size=0, name=None):
+def shifted_window_attention(inputs, window_size, num_heads=4, shift_size=0, pos_scale=-1, name=None):
     input_channel = inputs.shape[-1]
     window_size = window_size if isinstance(window_size, (list, tuple)) else [window_size, window_size]
     window_height = window_size[0] if window_size[0] < inputs.shape[1] else inputs.shape[1]
@@ -189,7 +241,7 @@ def shifted_window_attention(inputs, window_size, num_heads=4, shift_size=0, nam
     nn = tf.reshape(nn, [-1, window_height, window_width, input_channel])  # [batch * patch_height * patch_width, window_height, window_width, input_channel]
 
     mask = WindowAttentionMask(inputs.shape[1], inputs.shape[2], window_height, window_width, shift_height, shift_width) if should_shift else None
-    nn = window_mhsa_with_pair_wise_positional_embedding(nn, num_heads=num_heads, mask=mask, name=name)
+    nn = window_mhsa_with_pair_wise_positional_embedding(nn, num_heads=num_heads, mask=mask, pos_scale=pos_scale, name=name)
 
     # window_reverse, merge windows
     # [batch * patch_height, patch_width, window_height, window_width * input_channel], limit transpose perm <= 4
@@ -211,18 +263,18 @@ def shifted_window_attention(inputs, window_size, num_heads=4, shift_size=0, nam
 
 
 def swin_transformer_block(
-    inputs, window_size, num_heads=4, shift_size=0, mlp_ratio=4, mlp_drop_rate=0, attn_drop_rate=0, drop_rate=0, layer_scale=-1, name=None
+    inputs, window_size, num_heads=4, shift_size=0, pos_scale=-1, mlp_ratio=4, mlp_drop_rate=0, attn_drop_rate=0, drop_rate=0, name=None
 ):
     input_channel = inputs.shape[-1]
-    attn = shifted_window_attention(inputs, window_size, num_heads, shift_size, name=name + "attn_")
-    attn = layer_norm(attn, name=name + "attn_")
-    attn = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name + "1_gamma")(attn) if layer_scale >= 0 else attn
+    attn = shifted_window_attention(inputs, window_size, num_heads, shift_size, pos_scale=pos_scale, name=name + "attn_")
+    attn = layer_norm(attn, zero_gamma=True, name=name + "attn_")
+    # attn = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name + "1_gamma")(attn) if layer_scale >= 0 else attn
     attn = drop_block(attn, drop_rate=drop_rate, name=name + "attn_")
     attn_out = keras.layers.Add(name=name + "attn_out")([inputs, attn])
 
     mlp = mlp_block(attn_out, int(input_channel * mlp_ratio), drop_rate=mlp_drop_rate, use_conv=False, activation="gelu", name=name + "mlp_")
-    mlp = layer_norm(mlp, name=name + "mlp_")
-    mlp = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name + "2_gamma")(mlp) if layer_scale >= 0 else mlp
+    mlp = layer_norm(mlp, zero_gamma=True, name=name + "mlp_")
+    # mlp = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name + "2_gamma")(mlp) if layer_scale >= 0 else mlp
     mlp = drop_block(mlp, drop_rate=drop_rate, name=name + "mlp_")
     return keras.layers.Add(name=name + "output")([attn_out, mlp])
 
@@ -238,8 +290,8 @@ def patch_merging(inputs, name=""):
     nn = tf.reshape(inputs, [-1, 2, inputs.shape[2], input_channel])  # [batch * inputs.shape[1] // 2, height 2, inputs.shape[2], input_channel]
     nn = tf.transpose(nn, [0, 2, 1, 3])  # [batch * inputs.shape[1] // 2, inputs.shape[2], height 2, input_channel]
     nn = tf.reshape(nn, [-1, inputs.shape[1] // 2, inputs.shape[2] // 2, 2 * 2 * input_channel])
-    nn = layer_norm(nn, name=name)
     nn = keras.layers.Dense(2 * input_channel, use_bias=False, name=name + "dense")(nn)
+    nn = layer_norm(nn, name=name)
     return nn
 
 
@@ -247,11 +299,11 @@ def SwinTransformerV2(
     num_blocks=[2, 2, 6, 2],
     num_heads=[3, 6, 12, 24],
     embed_dim=96,
-    window_ratio=32,
+    window_size=7,
+    pos_scale=-1,  # If pretrained weights are from different input_shape or window_size, pos_scale is previous actually using window_size
     stem_patch_size=4,
     use_stack_norm=False,  # True for extra layer_norm on each stack end
     extra_norm_period=0,  # > 0 for extra layer_norm frequency in each stack. May combine with use_stack_norm=True
-    layer_scale=-1,
     input_shape=(224, 224, 3),
     num_classes=1000,
     drop_connect_rate=0,
@@ -265,8 +317,7 @@ def SwinTransformerV2(
     inputs = keras.layers.Input(input_shape)
     nn = keras.layers.Conv2D(embed_dim, kernel_size=stem_patch_size, strides=stem_patch_size, use_bias=True, name="stem_conv")(inputs)
     nn = layer_norm(nn, name="stem_")
-    # window_size = [input_shape[0] // window_ratio, input_shape[1] // window_ratio]
-    window_size = [int(tf.math.ceil(input_shape[0] / window_ratio)), int(tf.math.ceil(input_shape[1] / window_ratio))]
+    window_size = window_size if isinstance(window_size, (list, tuple)) else [window_size, window_size]
 
     """ stages """
     total_blocks = sum(num_blocks)
@@ -275,12 +326,13 @@ def SwinTransformerV2(
         stack_name = "stack{}_".format(stack_id + 1)
         if stack_id > 0:
             # height, width downsample * 0.5, channel upsample * 2
-            nn = patch_merging(nn, name=stack_name + "downsample")
+            nn = patch_merging(nn, name=stack_name + "downsample_")
+        cur_pos_scale = pos_scale[stack_id] if isinstance(pos_scale, (list, tuple)) else pos_scale
         for block_id in range(num_block):
             block_name = stack_name + "block{}_".format(block_id + 1)
             block_drop_rate = drop_connect_rate * global_block_id / total_blocks
             shift_size = 0 if block_id % 2 == 0 else 0.5
-            nn = swin_transformer_block(nn, window_size, num_head, shift_size, drop_rate=block_drop_rate, layer_scale=layer_scale, name=block_name)
+            nn = swin_transformer_block(nn, window_size, num_head, shift_size, cur_pos_scale, drop_rate=block_drop_rate, name=block_name)
             global_block_id += 1
             if extra_norm_period > 0 and (block_id + 1) % extra_norm_period == 0 and not (use_stack_norm and block_id == num_block - 1):
                 nn = layer_norm(nn, name=block_name + "output_")
@@ -295,43 +347,91 @@ def SwinTransformerV2(
     return model
 
 
-def SwinTransformerV2Tiny(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained=None, **kwargs):
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_tiny", **kwargs)
+def SwinTransformerV2Tiny_window8(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
+    window_size = kwargs.pop("window_size", 8)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_tiny_window8", **kwargs)
 
 
-def SwinTransformerV2Tiny_ns(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
-    use_stack_norm = True
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_tiny_ns", **kwargs)
+def SwinTransformerV2Tiny_window16(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
+    window_size = kwargs.pop("window_size", 16)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_tiny_window16", **kwargs)
 
 
-def SwinTransformerV2Small(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
+def SwinTransformerV2Small_window8(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [2, 2, 18, 2]
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_small", **kwargs)
+    window_size = kwargs.pop("window_size", 8)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_small_window8", **kwargs)
 
 
-def SwinTransformerV2Small_ns(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
+def SwinTransformerV2Small_window16(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [2, 2, 18, 2]
-    use_stack_norm = True
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_small_ns", **kwargs)
+    window_size = kwargs.pop("window_size", 16)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_small_window16", **kwargs)
 
 
-def SwinTransformerV2Base(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained=None, **kwargs):
+def SwinTransformerV2Base_window8(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
     num_blocks = [2, 2, 18, 2]
     num_heads = [4, 8, 16, 32]
     embed_dim = 128
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_base", **kwargs)
+    window_size = kwargs.pop("window_size", 8)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_base_window8", **kwargs)
 
 
-def SwinTransformerV2Large(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained=None, **kwargs):
+def SwinTransformerV2Base_window12(input_shape=(192, 192, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet21k", **kwargs):
+    num_blocks = [2, 2, 18, 2]
+    num_heads = [4, 8, 16, 32]
+    embed_dim = 128
+    window_size = kwargs.pop("window_size", 12)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_base_window12", **kwargs)
+
+
+def SwinTransformerV2Base_window16(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet", **kwargs):
+    num_blocks = [2, 2, 18, 2]
+    num_heads = [4, 8, 16, 32]
+    embed_dim = 128
+    window_size = kwargs.pop("window_size", 16)
+    pos_scale = kwargs.pop("pos_scale", [12, 12, 12, 6] if pretrained == "imagenet22k" else -1)  # 22k model is fine-tuned from window12
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_base_window16", **kwargs)
+
+
+def SwinTransformerV2Base_window24(input_shape=(384, 384, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet22k", **kwargs):
+    num_blocks = [2, 2, 18, 2]
+    num_heads = [4, 8, 16, 32]
+    embed_dim = 128
+    window_size = kwargs.pop("window_size", 24)
+    pos_scale = kwargs.pop("pos_scale", [12, 12, 12, 6] if pretrained == "imagenet22k" else -1)  # 22k model is fine-tuned from window12
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_base_window24", **kwargs)
+
+
+def SwinTransformerV2Large_window12(input_shape=(192, 192, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet21k", **kwargs):
     num_blocks = [2, 2, 18, 2]
     num_heads = [6, 12, 24, 48]
     embed_dim = 192
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_large", **kwargs)
+    window_size = kwargs.pop("window_size", 12)
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_large_window12", **kwargs)
 
 
-def SwinTransformerV2Giant(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained=None, **kwargs):
-    num_blocks = [2, 2, 42, 2]
-    num_heads = [16, 32, 64, 128]
-    embed_dim = 512
-    extra_norm_period = 6
-    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_giant", **kwargs)
+def SwinTransformerV2Large_window16(input_shape=(256, 256, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet22k", **kwargs):
+    num_blocks = [2, 2, 18, 2]
+    num_heads = [6, 12, 24, 48]
+    embed_dim = 192
+    window_size = kwargs.pop("window_size", 16)
+    pos_scale = kwargs.pop("pos_scale", [12, 12, 12, 6] if pretrained == "imagenet22k" else -1)  # 22k model is fine-tuned from window12
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_large_window16", **kwargs)
+
+
+def SwinTransformerV2Large_window24(input_shape=(384, 384, 3), num_classes=1000, classifier_activation="softmax", pretrained="imagenet22k", **kwargs):
+    num_blocks = [2, 2, 18, 2]
+    num_heads = [6, 12, 24, 48]
+    embed_dim = 192
+    window_size = kwargs.pop("window_size", 24)
+    pos_scale = kwargs.pop("pos_scale", [12, 12, 12, 6] if pretrained == "imagenet22k" else -1)  # 22k model is fine-tuned from window12
+    return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_large_window24", **kwargs)
+
+
+# def SwinTransformerV2Giant(input_shape=(224, 224, 3), num_classes=1000, classifier_activation="softmax", pretrained=None, **kwargs):
+#     num_blocks = [2, 2, 42, 2]
+#     num_heads = [16, 32, 64, 128]
+#     embed_dim = 512
+#     extra_norm_period = 6
+#     return SwinTransformerV2(**locals(), model_name="swin_transformer_v2_giant", **kwargs)
